// CkCharset.h: interface for the CkCharset class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _CKCHARSET_H
#define _CKCHARSET_H

#pragma once


class CkString;
class CkByteData;
#include "CkObject.h"


// CLASS: CkCharset
class CkCharset  : public CkObject
{
    private:
	void *m_impl;
	bool m_utf8;	// If true, all input "const char *" parameters are utf-8, otherwise they are ANSI strings.

	// Don't allow assignment or copying these objects.
	CkCharset(const CkCharset &) { } 
	CkCharset &operator=(const CkCharset &) { return *this; }

    public:
	CkCharset();
	virtual ~CkCharset();

	// BEGIN PUBLIC INTERFACE
	bool get_Utf8(void) const;
	void put_Utf8(bool b);


	bool GetHtmlFileCharset(const char *htmlFilename, CkString &strCharset);
	bool GetHtmlCharset(const CkByteData &htmlData, CkString &strCharset);
	bool GetHtmlCharset(const unsigned char *htmlData, unsigned long htmlDataLen, CkString &strCharset);
	bool ConvertHtmlFile(const char *inFilename, const char *outFilename);
	bool ConvertHtml(const CkByteData &htmlIn, CkByteData &htmlOut);
	bool ConvertHtml(const unsigned char *htmlIn, unsigned long htmlInLen, CkByteData &htmlOut);

	bool get_LastOutputAsQP(CkString &str);
	bool get_LastInputAsQP(CkString &str);
	bool get_LastOutputAsHex(CkString &str);
	bool get_LastInputAsHex(CkString &str);
	void put_SaveLast(bool value);
	bool get_SaveLast(void);

	bool WriteFile(const char *filename, const CkByteData &dataBuf);
	bool WriteFile(const char *filename, const unsigned char *data, unsigned long dataLen);
	bool ReadFile(const char *filename, CkByteData &dataBuf);

	bool ConvertFromUnicode(const CkByteData &uniData, CkByteData &mbData);
	bool ConvertToUnicode(const CkByteData &mbData, CkByteData &uniData);

	bool ConvertFromUnicode(const unsigned char *data, unsigned long dataLen, CkByteData &mbData);
	bool ConvertToUnicode(const unsigned char *data, unsigned long dataLen, CkByteData &uniData);

	bool VerifyFile(const char *charset, const char *filename);
	bool VerifyData(const char *charset, const CkByteData &charData);

	bool ConvertFile(const char *inFilename, const char *outFilename);

	bool ConvertData(const CkByteData &inData, CkByteData &outData);
	bool ConvertData(const unsigned char *inData, unsigned long inDataLen, CkByteData &outData);

	// Convert byte data into the memory buffer you provide. outDataLen is set
	// to the size of the output buffer 
	bool ConvertData(const unsigned char *inData, unsigned long inDataLen,
	    unsigned char *outData, unsigned long *outDataLen);

	bool get_ToCharset(CkString &str);
	bool put_ToCharset(const char *charset);
	bool get_FromCharset(CkString &str);
	bool put_FromCharset(const char *charset);
	bool get_Version(CkString &str);

	bool UnlockComponent(const char *unlockCode);
	bool IsUnlocked(void) const;

	// Error log retrieval and saving (these methods are common to all Chilkat VC++ classes.)
	bool SaveLastError(const char *filename);
        void LastErrorXml(CkString &str);
        void LastErrorHtml(CkString &str);
        void LastErrorText(CkString &str);

	int CharsetToCodePage(const char *charsetName);
	bool CodePageToCharset(int codePage, CkString &sCharset);

	// Error actions:
	// 0: drop error character.
	// 1: substitute default bytes
	// 2: hex numeric escape &#xXXXX;          
	// 3: RESERVED
	// 4: RESERVED
	// 5: RESERVED
	// 6: use alternate To charset
	// 7: pass through non-convertable bytes unchanged.
	int get_ErrorAction(void);
	void put_ErrorAction(int val);

	void get_AltToCharset(CkString &str);
	void put_AltToCharset(const char *charsetName);

	void SetErrorBytes(const unsigned char *data, unsigned long dataLen);
	void SetErrorString(const char *str);


	// END PUBLIC INTERFACE

	//bool DownloadHtml(const char *url, const char *proxy, const char *username, const char *password, CkByteData &htmlOut);
	//bool DetectCharset(const CkByteData &dataBuf, CkString &strCharset);
	//bool DetectCharset(const unsigned char *data, unsigned long dataLen, CkString &strCharset);
	//bool ConvertUtf8Data(const CkByteData &inData, CkByteData &outData, int errorOption, unsigned char errorChar);


};

#endif
